<?php

namespace Drp\UserManagement\Test\Feature;

use Carbon\Carbon;
use Drp\UserManagement\Contracts\TokenGenerator;
use Drp\UserManagement\Test\TestCase;
use Drp\UserManagement\User;
use Drp\UserManagement\UserCreator;
use Tests\Support\Fakes\FakeTokenGenerator;

Class AuthTest extends TestCase
{
    /**
     * @test
     */
    public function a_user_can_login_using_a_login_link()
    {
        $this->withoutEvents();

        $user = factory(User::class)->create([
            'email' => 'email',
            'first_login_token' => 'TOKEN',
            'login_link_expiry' => Carbon::now()->addDay(),
            'password' => null
        ]);

        $this->get('/login/INVALIDTOKEN')
            ->assertResponseStatus(500);

        $this->visitRoute('login.first', $user->first_login_token)
            ->seeRouteIs('account.setup');
    }

    /**
     * @test
     */
    public function a_user_must_setup_their_password_on_first_login()
    {
        $this->withoutEvents();

        $user = factory(User::class)
            ->create([
                'role' => 'Administrator',
                'email' => 'email',
                'first_login_token' => 'TOKEN',
                'login_link_expiry' => Carbon::now()->addDay(),
                'password' => null
            ]);

        $this->visit('/login/' . $user->first_login_token)
            ->seeRouteIs('account.setup')
            ->type('password', 'password')
            ->type('password', 'password_confirmation')
            ->press('Create')
            ->seeRouteIs('admin.index')
            ->seeSuccessAlert()
            ->seeText(trans('userManagement::users.setup'));

        $user = User::firstOrFail();
        $this->assertNotNull($user->password);
    }

    /**
     * @test
     */
    public function skip_initial_setup_when_password_is_present()
    {
        $this->withoutEvents();
        $user = factory(User::class)
            ->make(['password' => 'password']);
        $user->save();

        $user = $this->app->make(UserCreator::class)->reIssueLoginCode($user);

        $this->visitRoute('login.first', $user->first_login_token)
            ->seeRouteIs('admin.index');
    }

    /**
     * @test
     */
    public function a_user_cannot_login_if_there_token_has_expired()
    {
        $this->withoutEvents();

        $user = factory(User::class)->create([
            'email' => 'email',
            'first_login_token' => 'TOKEN',
            'login_link_expiry' => Carbon::now()->subDay(),
            'password' => null
        ]);


        $this->visit('/login/TOKEN')
            ->seeText('expired');

        $user->login_link_expiry = Carbon::now()->subWeeks(2);
        $user->save();

        $this->visit('/login/TOKEN')
            ->seeText(trans('userManagement::users.expired'));
    }

    /**
     * @test
     */
    public function a_user_cannot_login_if_they_are_not_setup()
    {
        $this->withoutEvents();

        $user = factory(User::class)->create([
            'email' => 'email',
            'first_login_token' => 'TOKEN',
            'login_link_expiry' => Carbon::now()->addDay(),
            'password' => null
        ]);

        $this->visit('/login/TOKEN')
            ->visit('/admin')
            ->seePageIs('/setup');
    }

    /**
     * @test
     */
    public function a_user_cannot_login_via_the_token_if_it_has_already_been_used()
    {
        $this->withoutEvents();

        $user = factory(User::class)
            ->create([
                'email' => 'email',
                'first_login_token' => 'TOKEN',
                'login_link_expiry' => Carbon::now()->addDay(),
                'password' => null
            ]);


        $this->visit('/login/' . $user->first_login_token)
            ->seeRouteIs('account.setup');

        $this->visit('/login/' . $user->first_login_token)
            ->seeRouteIs('account.setup')
            ->submitForm('Create', ['password' => 'password', 'password_confirmation' => 'password']);


        $this->get('/login/' . $user->first_login_token)
            ->assertResponseStatus(500);
    }
}
